"""Oracle contract binding and data structures."""

# This module has been generated using pyabigen v0.2.11

import typing
from dataclasses import dataclass

import eth_typing
import web3
from web3.contract import contract

__version__ = "v1.0.2-alpha"


@dataclass
class Config:
    """Port of `struct Config` on the Oracle contract."""

    autonity: eth_typing.ChecksumAddress
    operator: eth_typing.ChecksumAddress
    vote_period: int
    outlier_detection_threshold: int
    outlier_slashing_threshold: int
    base_slashing_rate: int


@dataclass
class RoundData:
    """Port of `struct RoundData` on the IOracle contract."""

    round: int
    price: int
    timestamp: int
    success: bool


class Oracle:
    """Oracle contract binding.

    This contract implements the Oracle for the Autonity Protocol, allowing voters to
    submit price reports and aggregate them while detecting outliers.

    Parameters
    ----------
    w3 : web3.Web3
    address : eth_typing.ChecksumAddress
        The address of a deployed Oracle contract.
    """

    _contract: contract.Contract

    def __init__(
        self,
        w3: web3.Web3,
        address: eth_typing.ChecksumAddress,
    ):
        self._contract = w3.eth.contract(
            address=address,
            abi=ABI,
        )

    @property
    def NewRound(self) -> contract.ContractEvent:
        """Binding for `event NewRound` on the Oracle contract."""
        return self._contract.events.NewRound

    @property
    def NewSymbols(self) -> contract.ContractEvent:
        """Binding for `event NewSymbols` on the Oracle contract."""
        return self._contract.events.NewSymbols

    @property
    def Penalized(self) -> contract.ContractEvent:
        """Binding for `event Penalized` on the Oracle contract."""
        return self._contract.events.Penalized

    @property
    def Voted(self) -> contract.ContractEvent:
        """Binding for `event Voted` on the Oracle contract."""
        return self._contract.events.Voted

    def config(
        self,
    ) -> Config:
        """Binding for `config` on the Oracle contract.

        Returns
        -------
        Config
        """
        return_value = self._contract.functions.config().call()
        return Config(
            eth_typing.ChecksumAddress(return_value[0]),
            eth_typing.ChecksumAddress(return_value[1]),
            int(return_value[2]),
            int(return_value[3]),
            int(return_value[4]),
            int(return_value[5]),
        )

    def distribute_rewards(
        self,
        _ntn: int,
    ) -> contract.ContractFunction:
        """Binding for `distributeRewards` on the Oracle contract.

        Parameters
        ----------
        _ntn : int

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.distributeRewards(
            _ntn,
        )

    def get_decimals(
        self,
    ) -> int:
        """Binding for `getDecimals` on the Oracle contract.

        Decimal places to be used with price reports

        Returns
        -------
        int
        """
        return_value = self._contract.functions.getDecimals().call()
        return int(return_value)

    def get_new_voters(
        self,
    ) -> typing.List[eth_typing.ChecksumAddress]:
        """Binding for `getNewVoters` on the Oracle contract.

        Retrieve the list of new participants in the Oracle process.

        Returns
        -------
        typing.List[eth_typing.ChecksumAddress]
        """
        return_value = self._contract.functions.getNewVoters().call()
        return [
            eth_typing.ChecksumAddress(return_value_elem)
            for return_value_elem in return_value
        ]

    def get_reward_period_performance(
        self,
        _voter: eth_typing.ChecksumAddress,
    ) -> int:
        """Binding for `getRewardPeriodPerformance` on the Oracle contract.

        Retrieve the performance for a voter in this reward (epoch) period.

        Parameters
        ----------
        _voter : eth_typing.ChecksumAddress

        Returns
        -------
        int
        """
        return_value = self._contract.functions.getRewardPeriodPerformance(
            _voter,
        ).call()
        return int(return_value)

    def get_round(
        self,
    ) -> int:
        """Binding for `getRound` on the Oracle contract.

        Retrieve the current round ID.

        Returns
        -------
        int
        """
        return_value = self._contract.functions.getRound().call()
        return int(return_value)

    def get_round_data(
        self,
        _round: int,
        _symbol: str,
    ) -> RoundData:
        """Binding for `getRoundData` on the Oracle contract.

        Return price data for a specific round.

        Parameters
        ----------
        _round : int
            , the round for which the price should be returned.
        _symbol : str
            , the symbol for which the current price should be returned.

        Returns
        -------
        RoundData
        """
        return_value = self._contract.functions.getRoundData(
            _round,
            _symbol,
        ).call()
        return RoundData(
            int(return_value[0]),
            int(return_value[1]),
            int(return_value[2]),
            bool(return_value[3]),
        )

    def get_symbols(
        self,
    ) -> typing.List[str]:
        """Binding for `getSymbols` on the Oracle contract.

        Retrieve the lists of symbols to be voted on.

        Returns
        -------
        typing.List[str]
        """
        return_value = self._contract.functions.getSymbols().call()
        return [str(return_value_elem) for return_value_elem in return_value]

    def get_vote_period(
        self,
    ) -> int:
        """Binding for `getVotePeriod` on the Oracle contract.

        Retrieve the vote period.

        Returns
        -------
        int
        """
        return_value = self._contract.functions.getVotePeriod().call()
        return int(return_value)

    def get_voters(
        self,
    ) -> typing.List[eth_typing.ChecksumAddress]:
        """Binding for `getVoters` on the Oracle contract.

        Retrieve the list of participants in the Oracle process.

        Returns
        -------
        typing.List[eth_typing.ChecksumAddress]
        """
        return_value = self._contract.functions.getVoters().call()
        return [
            eth_typing.ChecksumAddress(return_value_elem)
            for return_value_elem in return_value
        ]

    def last_round_block(
        self,
    ) -> int:
        """Binding for `lastRoundBlock` on the Oracle contract.

        Returns
        -------
        int
        """
        return_value = self._contract.functions.lastRoundBlock().call()
        return int(return_value)

    def latest_round_data(
        self,
        _symbol: str,
    ) -> RoundData:
        """Binding for `latestRoundData` on the Oracle contract.

        Return latest available price data.

        Parameters
        ----------
        _symbol : str
            , the symbol from which the current price should be returned.

        Returns
        -------
        RoundData
        """
        return_value = self._contract.functions.latestRoundData(
            _symbol,
        ).call()
        return RoundData(
            int(return_value[0]),
            int(return_value[1]),
            int(return_value[2]),
            bool(return_value[3]),
        )

    def reports(
        self,
        key0: str,
        key1: eth_typing.ChecksumAddress,
    ) -> typing.Tuple[int, int]:
        """Binding for `reports` on the Oracle contract.

        Parameters
        ----------
        key0 : str
        key1 : eth_typing.ChecksumAddress

        Returns
        -------
        int
        int
        """
        return_value = self._contract.functions.reports(
            key0,
            key1,
        ).call()
        return (
            int(return_value[0]),
            int(return_value[1]),
        )

    def set_slashing_config(
        self,
        _outlier_slashing_threshold: int,
        _outlier_detection_threshold: int,
        _base_slashing_rate: int,
    ) -> contract.ContractFunction:
        """Binding for `setSlashingConfig` on the Oracle contract.

        Setter for the internal slashing and outlier detection configuration.

        Parameters
        ----------
        _outlier_slashing_threshold : int
        _outlier_detection_threshold : int
        _base_slashing_rate : int

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.setSlashingConfig(
            _outlier_slashing_threshold,
            _outlier_detection_threshold,
            _base_slashing_rate,
        )

    def set_symbols(
        self,
        _symbols: typing.List[str],
    ) -> contract.ContractFunction:
        """Binding for `setSymbols` on the Oracle contract.

        Update the symbols to be requested. Only effective at the next round. Restricted
        to the operator account.

        Parameters
        ----------
        _symbols : typing.List[str]
            list of string symbols to be used. E.g. "ATN-USD"

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.setSymbols(
            _symbols,
        )

    def set_vote_period(
        self,
        _vote_period: int,
    ) -> contract.ContractFunction:
        """Binding for `setVotePeriod` on the Oracle contract.

        Setter for the vote period.

        Parameters
        ----------
        _vote_period : int

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.setVotePeriod(
            _vote_period,
        )

    def symbol_updated_round(
        self,
    ) -> int:
        """Binding for `symbolUpdatedRound` on the Oracle contract.

        Returns
        -------
        int
        """
        return_value = self._contract.functions.symbolUpdatedRound().call()
        return int(return_value)

    def update_voters(
        self,
    ) -> contract.ContractFunction:
        """Binding for `updateVoters` on the Oracle contract.

        Called when the previous round is ended. Updates the voter info for new voters.

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.updateVoters()

    def voter_info(
        self,
        key0: eth_typing.ChecksumAddress,
    ) -> typing.Tuple[int, int, int, bool, bool]:
        """Binding for `voterInfo` on the Oracle contract.

        Parameters
        ----------
        key0 : eth_typing.ChecksumAddress

        Returns
        -------
        int
        int
        int
        bool
        bool
        """
        return_value = self._contract.functions.voterInfo(
            key0,
        ).call()
        return (
            int(return_value[0]),
            int(return_value[1]),
            int(return_value[2]),
            bool(return_value[3]),
            bool(return_value[4]),
        )

    def voter_treasuries(
        self,
        key0: eth_typing.ChecksumAddress,
    ) -> eth_typing.ChecksumAddress:
        """Binding for `voterTreasuries` on the Oracle contract.

        Parameters
        ----------
        key0 : eth_typing.ChecksumAddress

        Returns
        -------
        eth_typing.ChecksumAddress
        """
        return_value = self._contract.functions.voterTreasuries(
            key0,
        ).call()
        return eth_typing.ChecksumAddress(return_value)

    def voter_validators(
        self,
        key0: eth_typing.ChecksumAddress,
    ) -> eth_typing.ChecksumAddress:
        """Binding for `voterValidators` on the Oracle contract.

        Parameters
        ----------
        key0 : eth_typing.ChecksumAddress

        Returns
        -------
        eth_typing.ChecksumAddress
        """
        return_value = self._contract.functions.voterValidators(
            key0,
        ).call()
        return eth_typing.ChecksumAddress(return_value)


ABI = typing.cast(
    eth_typing.ABI,
    [
        {
            "inputs": [
                {"internalType": "address[]", "name": "_voters", "type": "address[]"},
                {
                    "internalType": "address[]",
                    "name": "_nodeAddresses",
                    "type": "address[]",
                },
                {
                    "internalType": "address[]",
                    "name": "_treasuries",
                    "type": "address[]",
                },
                {"internalType": "string[]", "name": "_symbols", "type": "string[]"},
                {
                    "components": [
                        {
                            "internalType": "contract Autonity",
                            "name": "autonity",
                            "type": "address",
                        },
                        {
                            "internalType": "address",
                            "name": "operator",
                            "type": "address",
                        },
                        {
                            "internalType": "uint256",
                            "name": "votePeriod",
                            "type": "uint256",
                        },
                        {
                            "internalType": "int256",
                            "name": "outlierDetectionThreshold",
                            "type": "int256",
                        },
                        {
                            "internalType": "int256",
                            "name": "outlierSlashingThreshold",
                            "type": "int256",
                        },
                        {
                            "internalType": "uint256",
                            "name": "baseSlashingRate",
                            "type": "uint256",
                        },
                    ],
                    "internalType": "struct Oracle.Config",
                    "name": "_config",
                    "type": "tuple",
                },
            ],
            "stateMutability": "nonpayable",
            "type": "constructor",
        },
        {
            "anonymous": False,
            "inputs": [
                {
                    "indexed": False,
                    "internalType": "uint256",
                    "name": "_round",
                    "type": "uint256",
                },
                {
                    "indexed": False,
                    "internalType": "uint256",
                    "name": "_height",
                    "type": "uint256",
                },
                {
                    "indexed": False,
                    "internalType": "uint256",
                    "name": "_timestamp",
                    "type": "uint256",
                },
                {
                    "indexed": False,
                    "internalType": "uint256",
                    "name": "_votePeriod",
                    "type": "uint256",
                },
            ],
            "name": "NewRound",
            "type": "event",
        },
        {
            "anonymous": False,
            "inputs": [
                {
                    "indexed": False,
                    "internalType": "string[]",
                    "name": "_symbols",
                    "type": "string[]",
                },
                {
                    "indexed": False,
                    "internalType": "uint256",
                    "name": "_round",
                    "type": "uint256",
                },
            ],
            "name": "NewSymbols",
            "type": "event",
        },
        {
            "anonymous": False,
            "inputs": [
                {
                    "indexed": True,
                    "internalType": "address",
                    "name": "_participant",
                    "type": "address",
                },
                {
                    "indexed": False,
                    "internalType": "string",
                    "name": "_symbol",
                    "type": "string",
                },
                {
                    "indexed": False,
                    "internalType": "int256",
                    "name": "_median",
                    "type": "int256",
                },
                {
                    "indexed": False,
                    "internalType": "uint120",
                    "name": "_reported",
                    "type": "uint120",
                },
            ],
            "name": "Penalized",
            "type": "event",
        },
        {
            "anonymous": False,
            "inputs": [
                {
                    "indexed": True,
                    "internalType": "address",
                    "name": "_voter",
                    "type": "address",
                },
                {
                    "indexed": False,
                    "internalType": "int256[]",
                    "name": "_votes",
                    "type": "int256[]",
                },
            ],
            "name": "Voted",
            "type": "event",
        },
        {"stateMutability": "payable", "type": "fallback"},
        {
            "inputs": [],
            "name": "config",
            "outputs": [
                {
                    "internalType": "contract Autonity",
                    "name": "autonity",
                    "type": "address",
                },
                {"internalType": "address", "name": "operator", "type": "address"},
                {"internalType": "uint256", "name": "votePeriod", "type": "uint256"},
                {
                    "internalType": "int256",
                    "name": "outlierDetectionThreshold",
                    "type": "int256",
                },
                {
                    "internalType": "int256",
                    "name": "outlierSlashingThreshold",
                    "type": "int256",
                },
                {
                    "internalType": "uint256",
                    "name": "baseSlashingRate",
                    "type": "uint256",
                },
            ],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [{"internalType": "uint256", "name": "_ntn", "type": "uint256"}],
            "name": "distributeRewards",
            "outputs": [],
            "stateMutability": "payable",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "finalize",
            "outputs": [{"internalType": "bool", "name": "", "type": "bool"}],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "getDecimals",
            "outputs": [{"internalType": "uint8", "name": "", "type": "uint8"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "getNewVoters",
            "outputs": [{"internalType": "address[]", "name": "", "type": "address[]"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "_voter", "type": "address"}
            ],
            "name": "getRewardPeriodPerformance",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "getRound",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "uint256", "name": "_round", "type": "uint256"},
                {"internalType": "string", "name": "_symbol", "type": "string"},
            ],
            "name": "getRoundData",
            "outputs": [
                {
                    "components": [
                        {"internalType": "uint256", "name": "round", "type": "uint256"},
                        {"internalType": "uint256", "name": "price", "type": "uint256"},
                        {
                            "internalType": "uint256",
                            "name": "timestamp",
                            "type": "uint256",
                        },
                        {"internalType": "bool", "name": "success", "type": "bool"},
                    ],
                    "internalType": "struct IOracle.RoundData",
                    "name": "data",
                    "type": "tuple",
                }
            ],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "getSymbols",
            "outputs": [{"internalType": "string[]", "name": "", "type": "string[]"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "getVotePeriod",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "getVoters",
            "outputs": [{"internalType": "address[]", "name": "", "type": "address[]"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "lastRoundBlock",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [{"internalType": "string", "name": "_symbol", "type": "string"}],
            "name": "latestRoundData",
            "outputs": [
                {
                    "components": [
                        {"internalType": "uint256", "name": "round", "type": "uint256"},
                        {"internalType": "uint256", "name": "price", "type": "uint256"},
                        {
                            "internalType": "uint256",
                            "name": "timestamp",
                            "type": "uint256",
                        },
                        {"internalType": "bool", "name": "success", "type": "bool"},
                    ],
                    "internalType": "struct IOracle.RoundData",
                    "name": "data",
                    "type": "tuple",
                }
            ],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "string", "name": "", "type": "string"},
                {"internalType": "address", "name": "", "type": "address"},
            ],
            "name": "reports",
            "outputs": [
                {"internalType": "uint120", "name": "price", "type": "uint120"},
                {"internalType": "uint8", "name": "confidence", "type": "uint8"},
            ],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "_operator", "type": "address"}
            ],
            "name": "setOperator",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {
                    "internalType": "int256",
                    "name": "_outlierSlashingThreshold",
                    "type": "int256",
                },
                {
                    "internalType": "int256",
                    "name": "_outlierDetectionThreshold",
                    "type": "int256",
                },
                {
                    "internalType": "uint256",
                    "name": "_baseSlashingRate",
                    "type": "uint256",
                },
            ],
            "name": "setSlashingConfig",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "string[]", "name": "_symbols", "type": "string[]"}
            ],
            "name": "setSymbols",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "uint256", "name": "_votePeriod", "type": "uint256"}
            ],
            "name": "setVotePeriod",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {
                    "internalType": "address[]",
                    "name": "_newVoters",
                    "type": "address[]",
                },
                {"internalType": "address[]", "name": "_treasury", "type": "address[]"},
                {
                    "internalType": "address[]",
                    "name": "_validator",
                    "type": "address[]",
                },
            ],
            "name": "setVoters",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "symbolUpdatedRound",
            "outputs": [{"internalType": "int256", "name": "", "type": "int256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "updateVoters",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "uint256", "name": "_commit", "type": "uint256"},
                {
                    "components": [
                        {"internalType": "uint120", "name": "price", "type": "uint120"},
                        {
                            "internalType": "uint8",
                            "name": "confidence",
                            "type": "uint8",
                        },
                    ],
                    "internalType": "struct IOracle.Report[]",
                    "name": "_reports",
                    "type": "tuple[]",
                },
                {"internalType": "uint256", "name": "_salt", "type": "uint256"},
                {"internalType": "uint8", "name": "_extra", "type": "uint8"},
            ],
            "name": "vote",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [{"internalType": "address", "name": "", "type": "address"}],
            "name": "voterInfo",
            "outputs": [
                {"internalType": "uint256", "name": "round", "type": "uint256"},
                {"internalType": "uint256", "name": "commit", "type": "uint256"},
                {"internalType": "uint256", "name": "performance", "type": "uint256"},
                {"internalType": "bool", "name": "isVoter", "type": "bool"},
                {"internalType": "bool", "name": "reportAvailable", "type": "bool"},
            ],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [{"internalType": "address", "name": "", "type": "address"}],
            "name": "voterTreasuries",
            "outputs": [{"internalType": "address", "name": "", "type": "address"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [{"internalType": "address", "name": "", "type": "address"}],
            "name": "voterValidators",
            "outputs": [{"internalType": "address", "name": "", "type": "address"}],
            "stateMutability": "view",
            "type": "function",
        },
        {"stateMutability": "payable", "type": "receive"},
    ],
)



"""Stabilization contract binding and data structures."""

# This module has been generated using pyabigen v0.2.11

import typing
from dataclasses import dataclass

import eth_typing
import web3
from plum import dispatch
from web3.contract import contract

__version__ = "v1.0.2-alpha"


@dataclass
class Config:
    """Port of `struct Config` on the Stabilization contract."""

    borrow_interest_rate: int
    liquidation_ratio: int
    min_collateralization_ratio: int
    min_debt_requirement: int
    target_price: int


class Stabilization:
    """Stabilization contract binding.

    A CDP-based stabilization mechanism for the Auton.

    Parameters
    ----------
    w3 : web3.Web3
    address : eth_typing.ChecksumAddress
        The address of a deployed Stabilization contract.
    """

    _contract: contract.Contract

    def __init__(
        self,
        w3: web3.Web3,
        address: eth_typing.ChecksumAddress,
    ):
        self._contract = w3.eth.contract(
            address=address,
            abi=ABI,
        )

    @property
    def Borrow(self) -> contract.ContractEvent:
        """Binding for `event Borrow` on the Stabilization contract.

        Auton was borrowed from a CDP
        """
        return self._contract.events.Borrow

    @property
    def Deposit(self) -> contract.ContractEvent:
        """Binding for `event Deposit` on the Stabilization contract.

        Collateral Token was deposited into a CDP
        """
        return self._contract.events.Deposit

    @property
    def Liquidate(self) -> contract.ContractEvent:
        """Binding for `event Liquidate` on the Stabilization contract.

        A CDP was liquidated
        """
        return self._contract.events.Liquidate

    @property
    def Repay(self) -> contract.ContractEvent:
        """Binding for `event Repay` on the Stabilization contract.

        Auton debt was paid into a CDP
        """
        return self._contract.events.Repay

    @property
    def Withdraw(self) -> contract.ContractEvent:
        """Binding for `event Withdraw` on the Stabilization contract.

        Collateral Token was withdrawn from a CDP
        """
        return self._contract.events.Withdraw

    def scale(
        self,
    ) -> int:
        """Binding for `SCALE` on the Stabilization contract.

        The decimal places in fixed-point integer representation.

        Returns
        -------
        int
        """
        return_value = self._contract.functions.SCALE().call()
        return int(return_value)

    def scale_factor(
        self,
    ) -> int:
        """Binding for `SCALE_FACTOR` on the Stabilization contract.

        The multiplier for scaling numbers to the required scale.

        Returns
        -------
        int
        """
        return_value = self._contract.functions.SCALE_FACTOR().call()
        return int(return_value)

    def seconds_in_year(
        self,
    ) -> int:
        """Binding for `SECONDS_IN_YEAR` on the Stabilization contract.

        A year is assumed to have 365 days for interest rate calculations.

        Returns
        -------
        int
        """
        return_value = self._contract.functions.SECONDS_IN_YEAR().call()
        return int(return_value)

    def accounts(
        self,
    ) -> typing.List[eth_typing.ChecksumAddress]:
        """Binding for `accounts` on the Stabilization contract.

        Retrieve all the accounts that have opened a CDP.

        Returns
        -------
        typing.List[eth_typing.ChecksumAddress]
            Array of CDP account addresses
        """
        return_value = self._contract.functions.accounts().call()
        return [
            eth_typing.ChecksumAddress(return_value_elem)
            for return_value_elem in return_value
        ]

    def borrow(
        self,
        amount: int,
    ) -> contract.ContractFunction:
        """Binding for `borrow` on the Stabilization contract.

        Borrow Auton against the CDP Collateral. The CDP must not be liquidatable, the
        `amount` must not exceed the borrow limit, the debt after borrowing must satisfy
        the minimum debt requirement.

        Parameters
        ----------
        amount : int
            Auton to borrow

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.borrow(
            amount,
        )

    def borrow_limit(
        self,
        collateral: int,
        price: int,
        target_price: int,
        mcr: int,
    ) -> int:
        """Binding for `borrowLimit` on the Stabilization contract.

        Calculate the maximum amount of Amount that can be borrowed for the given amount
        of Collateral Token.

        Parameters
        ----------
        collateral : int
            Amount of Collateral Token backing the debt
        price : int
            The price of Collateral Token in Auton
        target_price : int
        mcr : int
            The minimum collateralization ratio

        Returns
        -------
        int
            The maximum Auton that can be borrowed
        """
        return_value = self._contract.functions.borrowLimit(
            collateral,
            price,
            target_price,
            mcr,
        ).call()
        return int(return_value)

    def cdps(
        self,
        key0: eth_typing.ChecksumAddress,
    ) -> typing.Tuple[int, int, int, int]:
        """Binding for `cdps` on the Stabilization contract.

        A mapping to retrieve the CDP for an account address.

        Parameters
        ----------
        key0 : eth_typing.ChecksumAddress

        Returns
        -------
        int
        int
        int
        int
        """
        return_value = self._contract.functions.cdps(
            key0,
        ).call()
        return (
            int(return_value[0]),
            int(return_value[1]),
            int(return_value[2]),
            int(return_value[3]),
        )

    def collateral_price(
        self,
    ) -> int:
        """Binding for `collateralPrice` on the Stabilization contract.

        Price the Collateral Token in Auton. Retrieves the Collateral Token price from
        the Oracle Contract and converts it to Auton.

        Returns
        -------
        int
            Price of Collateral Token
        """
        return_value = self._contract.functions.collateralPrice().call()
        return int(return_value)

    def config(
        self,
    ) -> Config:
        """Binding for `config` on the Stabilization contract.

        The Config object that stores Stabilization Contract parameters.

        Returns
        -------
        Config
        """
        return_value = self._contract.functions.config().call()
        return Config(
            int(return_value[0]),
            int(return_value[1]),
            int(return_value[2]),
            int(return_value[3]),
            int(return_value[4]),
        )

    @dispatch  # type: ignore
    def debt_amount(  # type: ignore # noqa: F811
        self,
        account: eth_typing.ChecksumAddress,
        timestamp: int,
    ) -> int:
        """Binding for `debtAmount` on the Stabilization contract.

        Calculate the debt amount outstanding for a CDP at the given timestamp. The
        timestamp must be equal or later than the time of the CDP last borrow or
        repayment.

        Parameters
        ----------
        account : eth_typing.ChecksumAddress
            The CDP account address
        timestamp : int
            The timestamp to value the debt

        Returns
        -------
        int
            The debt amount
        """
        return_value = self._contract.functions.debtAmount(
            account,
            timestamp,
        ).call()
        return int(return_value)

    @dispatch  # type: ignore
    def debt_amount(  # type: ignore # noqa: F811
        self,
        account: eth_typing.ChecksumAddress,
    ) -> int:
        """Binding for `debtAmount` on the Stabilization contract.

        Calculate the current debt amount outstanding for a CDP.

        Parameters
        ----------
        account : eth_typing.ChecksumAddress
            The CDP account address

        Returns
        -------
        int
            The debt amount
        """
        return_value = self._contract.functions.debtAmount(
            account,
        ).call()
        return int(return_value)

    def deposit(
        self,
        amount: int,
    ) -> contract.ContractFunction:
        """Binding for `deposit` on the Stabilization contract.

        Deposit Collateral Token using the ERC20 allowance mechanism. Before calling
        this function, the CDP owner must approve the Stabilization contract to spend
        Collateral Token on their behalf for the full amount to be deposited.

        Parameters
        ----------
        amount : int
            Units of Collateral Token to deposit (non-zero)

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.deposit(
            amount,
        )

    def interest_due(
        self,
        debt: int,
        rate: int,
        time_borrow: int,
        time_due: int,
    ) -> int:
        """Binding for `interestDue` on the Stabilization contract.

        Calculate the interest due for a given amount of debt.

        Parameters
        ----------
        debt : int
            The debt amount
        rate : int
            The borrow interest rate
        time_borrow : int
        time_due : int

        Returns
        -------
        int
            @dev Makes use of the prb-math library for natural exponentiation.
        """
        return_value = self._contract.functions.interestDue(
            debt,
            rate,
            time_borrow,
            time_due,
        ).call()
        return int(return_value)

    def is_liquidatable(
        self,
        account: eth_typing.ChecksumAddress,
    ) -> bool:
        """Binding for `isLiquidatable` on the Stabilization contract.

        Determine if the CDP is currently liquidatable.

        Parameters
        ----------
        account : eth_typing.ChecksumAddress
            The CDP account address

        Returns
        -------
        bool
            Whether the CDP is liquidatable
        """
        return_value = self._contract.functions.isLiquidatable(
            account,
        ).call()
        return bool(return_value)

    def liquidate(
        self,
        account: eth_typing.ChecksumAddress,
    ) -> contract.ContractFunction:
        """Binding for `liquidate` on the Stabilization contract.

        Liquidate a CDP that is undercollateralized. The liquidator must pay all the CDP
        debt outstanding. As a reward, the liquidator will receive the collateral that
        is held in the CDP. The transaction value is the payment amount. After covering
        the CDP's debt, any surplus is refunded to the liquidator.

        Parameters
        ----------
        account : eth_typing.ChecksumAddress
            The CDP account address to liquidate

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.liquidate(
            account,
        )

    def minimum_collateral(
        self,
        principal: int,
        price: int,
        mcr: int,
    ) -> int:
        """Binding for `minimumCollateral` on the Stabilization contract.

        Calculate the minimum amount of Collateral Token that must be deposited in the
        CDP in order to borrow the given amount of Autons.

        Parameters
        ----------
        principal : int
            Auton amount to borrow
        price : int
            The price of Collateral Token in Auton
        mcr : int
            The minimum collateralization ratio

        Returns
        -------
        int
            The minimum Collateral Token amount required
        """
        return_value = self._contract.functions.minimumCollateral(
            principal,
            price,
            mcr,
        ).call()
        return int(return_value)

    def remove_cdp_restrictions(
        self,
    ) -> contract.ContractFunction:
        """Binding for `removeCDPRestrictions` on the Stabilization contract.

        Transition out of the restricted state.

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.removeCDPRestrictions()

    def repay(
        self,
    ) -> contract.ContractFunction:
        """Binding for `repay` on the Stabilization contract.

        Make a payment towards CDP debt. The transaction value is the payment amount.
        The debt after payment must satisfy the minimum debt requirement. The payment
        first covers the outstanding interest debt before the principal debt.

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.repay()

    def set_atn_supply_operator(
        self,
        atn_supply_operator: eth_typing.ChecksumAddress,
    ) -> contract.ContractFunction:
        """Binding for `setAtnSupplyOperator` on the Stabilization contract.

        Set the _atnSupplyOperator address.

        Parameters
        ----------
        atn_supply_operator : eth_typing.ChecksumAddress

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.setAtnSupplyOperator(
            atn_supply_operator,
        )

    def set_liquidation_ratio(
        self,
        ratio: int,
    ) -> contract.ContractFunction:
        """Binding for `setLiquidationRatio` on the Stabilization contract.

        Set the liquidation ratio. Must be less than the minimum collateralization
        ratio.

        Parameters
        ----------
        ratio : int
            The liquidation ratio

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.setLiquidationRatio(
            ratio,
        )

    def set_min_collateralization_ratio(
        self,
        ratio: int,
    ) -> contract.ContractFunction:
        """Binding for `setMinCollateralizationRatio` on the Stabilization contract.

        Set the minimum collateralization ratio. Must be positive and greater than the
        liquidation ratio.

        Parameters
        ----------
        ratio : int
            The minimum collateralization ratio

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.setMinCollateralizationRatio(
            ratio,
        )

    def set_min_debt_requirement(
        self,
        amount: int,
    ) -> contract.ContractFunction:
        """Binding for `setMinDebtRequirement` on the Stabilization contract.

        Set the minimum debt requirement.

        Parameters
        ----------
        amount : int
            The minimum debt amount

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.setMinDebtRequirement(
            amount,
        )

    def set_supply_control(
        self,
        supply_control: eth_typing.ChecksumAddress,
    ) -> contract.ContractFunction:
        """Binding for `setSupplyControl` on the Stabilization contract.

        Set the SupplyControl Contract address.

        Parameters
        ----------
        supply_control : eth_typing.ChecksumAddress

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.setSupplyControl(
            supply_control,
        )

    def under_collateralized(
        self,
        collateral: int,
        price: int,
        debt: int,
        liquidation_ratio: int,
    ) -> bool:
        """Binding for `underCollateralized` on the Stabilization contract.

        Determine if a debt position is undercollateralized.

        Parameters
        ----------
        collateral : int
            The collateral amount
        price : int
            The price of Collateral Token in Auton
        debt : int
            The debt amount
        liquidation_ratio : int

        Returns
        -------
        bool
            Whether the position is liquidatable
        """
        return_value = self._contract.functions.underCollateralized(
            collateral,
            price,
            debt,
            liquidation_ratio,
        ).call()
        return bool(return_value)

    def withdraw(
        self,
        amount: int,
    ) -> contract.ContractFunction:
        """Binding for `withdraw` on the Stabilization contract.

        Request a withdrawal of Collateral Token. The CDP must not be liquidatable and
        the withdrawal must not reduce the remaining Collateral Token amount below the
        minimum collateral amount.

        Parameters
        ----------
        amount : int
            Units of Collateral Token to withdraw

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.withdraw(
            amount,
        )


ABI = typing.cast(
    eth_typing.ABI,
    [
        {
            "inputs": [
                {
                    "components": [
                        {
                            "internalType": "uint256",
                            "name": "borrowInterestRate",
                            "type": "uint256",
                        },
                        {
                            "internalType": "uint256",
                            "name": "liquidationRatio",
                            "type": "uint256",
                        },
                        {
                            "internalType": "uint256",
                            "name": "minCollateralizationRatio",
                            "type": "uint256",
                        },
                        {
                            "internalType": "uint256",
                            "name": "minDebtRequirement",
                            "type": "uint256",
                        },
                        {
                            "internalType": "uint256",
                            "name": "targetPrice",
                            "type": "uint256",
                        },
                    ],
                    "internalType": "struct Stabilization.Config",
                    "name": "config_",
                    "type": "tuple",
                },
                {"internalType": "address", "name": "autonity", "type": "address"},
                {"internalType": "address", "name": "operator", "type": "address"},
                {"internalType": "address", "name": "oracle", "type": "address"},
                {"internalType": "address", "name": "supplyControl", "type": "address"},
                {
                    "internalType": "contract IERC20",
                    "name": "collateralToken",
                    "type": "address",
                },
            ],
            "stateMutability": "nonpayable",
            "type": "constructor",
        },
        {"inputs": [], "name": "InsufficientAllowance", "type": "error"},
        {"inputs": [], "name": "InsufficientCollateral", "type": "error"},
        {"inputs": [], "name": "InsufficientPayment", "type": "error"},
        {"inputs": [], "name": "InvalidAmount", "type": "error"},
        {"inputs": [], "name": "InvalidDebtPosition", "type": "error"},
        {"inputs": [], "name": "InvalidParameter", "type": "error"},
        {"inputs": [], "name": "InvalidPrice", "type": "error"},
        {"inputs": [], "name": "Liquidatable", "type": "error"},
        {"inputs": [], "name": "NoDebtPosition", "type": "error"},
        {"inputs": [], "name": "NotLiquidatable", "type": "error"},
        {
            "inputs": [
                {"internalType": "uint256", "name": "x", "type": "uint256"},
                {"internalType": "uint256", "name": "y", "type": "uint256"},
            ],
            "name": "PRBMath_MulDiv18_Overflow",
            "type": "error",
        },
        {
            "inputs": [
                {"internalType": "uint256", "name": "x", "type": "uint256"},
                {"internalType": "uint256", "name": "y", "type": "uint256"},
                {"internalType": "uint256", "name": "denominator", "type": "uint256"},
            ],
            "name": "PRBMath_MulDiv_Overflow",
            "type": "error",
        },
        {
            "inputs": [{"internalType": "UD60x18", "name": "x", "type": "uint256"}],
            "name": "PRBMath_UD60x18_Exp2_InputTooBig",
            "type": "error",
        },
        {
            "inputs": [{"internalType": "UD60x18", "name": "x", "type": "uint256"}],
            "name": "PRBMath_UD60x18_Exp_InputTooBig",
            "type": "error",
        },
        {"inputs": [], "name": "PriceUnavailable", "type": "error"},
        {"inputs": [], "name": "TransferFailed", "type": "error"},
        {"inputs": [], "name": "Unauthorized", "type": "error"},
        {"inputs": [], "name": "ZeroValue", "type": "error"},
        {
            "anonymous": False,
            "inputs": [
                {
                    "indexed": True,
                    "internalType": "address",
                    "name": "account",
                    "type": "address",
                },
                {
                    "indexed": False,
                    "internalType": "uint256",
                    "name": "amount",
                    "type": "uint256",
                },
            ],
            "name": "Borrow",
            "type": "event",
        },
        {
            "anonymous": False,
            "inputs": [
                {
                    "indexed": True,
                    "internalType": "address",
                    "name": "account",
                    "type": "address",
                },
                {
                    "indexed": False,
                    "internalType": "uint256",
                    "name": "amount",
                    "type": "uint256",
                },
            ],
            "name": "Deposit",
            "type": "event",
        },
        {
            "anonymous": False,
            "inputs": [
                {
                    "indexed": True,
                    "internalType": "address",
                    "name": "account",
                    "type": "address",
                },
                {
                    "indexed": False,
                    "internalType": "address",
                    "name": "liquidator",
                    "type": "address",
                },
            ],
            "name": "Liquidate",
            "type": "event",
        },
        {
            "anonymous": False,
            "inputs": [
                {
                    "indexed": True,
                    "internalType": "address",
                    "name": "account",
                    "type": "address",
                },
                {
                    "indexed": False,
                    "internalType": "uint256",
                    "name": "amount",
                    "type": "uint256",
                },
            ],
            "name": "Repay",
            "type": "event",
        },
        {
            "anonymous": False,
            "inputs": [
                {
                    "indexed": True,
                    "internalType": "address",
                    "name": "account",
                    "type": "address",
                },
                {
                    "indexed": False,
                    "internalType": "uint256",
                    "name": "amount",
                    "type": "uint256",
                },
            ],
            "name": "Withdraw",
            "type": "event",
        },
        {
            "inputs": [],
            "name": "SCALE",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "SCALE_FACTOR",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "SECONDS_IN_YEAR",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "accounts",
            "outputs": [{"internalType": "address[]", "name": "", "type": "address[]"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "uint256", "name": "amount", "type": "uint256"}
            ],
            "name": "borrow",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "uint256", "name": "collateral", "type": "uint256"},
                {"internalType": "uint256", "name": "price", "type": "uint256"},
                {"internalType": "uint256", "name": "targetPrice", "type": "uint256"},
                {"internalType": "uint256", "name": "mcr", "type": "uint256"},
            ],
            "name": "borrowLimit",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "pure",
            "type": "function",
        },
        {
            "inputs": [{"internalType": "address", "name": "", "type": "address"}],
            "name": "cdps",
            "outputs": [
                {"internalType": "uint256", "name": "timestamp", "type": "uint256"},
                {"internalType": "uint256", "name": "collateral", "type": "uint256"},
                {"internalType": "uint256", "name": "principal", "type": "uint256"},
                {"internalType": "uint256", "name": "interest", "type": "uint256"},
            ],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "collateralPrice",
            "outputs": [
                {"internalType": "uint256", "name": "price", "type": "uint256"}
            ],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "config",
            "outputs": [
                {
                    "internalType": "uint256",
                    "name": "borrowInterestRate",
                    "type": "uint256",
                },
                {
                    "internalType": "uint256",
                    "name": "liquidationRatio",
                    "type": "uint256",
                },
                {
                    "internalType": "uint256",
                    "name": "minCollateralizationRatio",
                    "type": "uint256",
                },
                {
                    "internalType": "uint256",
                    "name": "minDebtRequirement",
                    "type": "uint256",
                },
                {"internalType": "uint256", "name": "targetPrice", "type": "uint256"},
            ],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "account", "type": "address"},
                {"internalType": "uint256", "name": "timestamp", "type": "uint256"},
            ],
            "name": "debtAmount",
            "outputs": [{"internalType": "uint256", "name": "debt", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "account", "type": "address"}
            ],
            "name": "debtAmount",
            "outputs": [{"internalType": "uint256", "name": "debt", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "uint256", "name": "amount", "type": "uint256"}
            ],
            "name": "deposit",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "uint256", "name": "debt", "type": "uint256"},
                {"internalType": "uint256", "name": "rate", "type": "uint256"},
                {"internalType": "uint256", "name": "timeBorrow", "type": "uint256"},
                {"internalType": "uint256", "name": "timeDue", "type": "uint256"},
            ],
            "name": "interestDue",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "pure",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "account", "type": "address"}
            ],
            "name": "isLiquidatable",
            "outputs": [{"internalType": "bool", "name": "", "type": "bool"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "account", "type": "address"}
            ],
            "name": "liquidate",
            "outputs": [],
            "stateMutability": "payable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "uint256", "name": "principal", "type": "uint256"},
                {"internalType": "uint256", "name": "price", "type": "uint256"},
                {"internalType": "uint256", "name": "mcr", "type": "uint256"},
            ],
            "name": "minimumCollateral",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "pure",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "removeCDPRestrictions",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "repay",
            "outputs": [],
            "stateMutability": "payable",
            "type": "function",
        },
        {
            "inputs": [
                {
                    "internalType": "address",
                    "name": "atnSupplyOperator",
                    "type": "address",
                }
            ],
            "name": "setAtnSupplyOperator",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [{"internalType": "uint256", "name": "ratio", "type": "uint256"}],
            "name": "setLiquidationRatio",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [{"internalType": "uint256", "name": "ratio", "type": "uint256"}],
            "name": "setMinCollateralizationRatio",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "uint256", "name": "amount", "type": "uint256"}
            ],
            "name": "setMinDebtRequirement",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "operator", "type": "address"}
            ],
            "name": "setOperator",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "oracle", "type": "address"}
            ],
            "name": "setOracle",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "supplyControl", "type": "address"}
            ],
            "name": "setSupplyControl",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "uint256", "name": "collateral", "type": "uint256"},
                {"internalType": "uint256", "name": "price", "type": "uint256"},
                {"internalType": "uint256", "name": "debt", "type": "uint256"},
                {
                    "internalType": "uint256",
                    "name": "liquidationRatio",
                    "type": "uint256",
                },
            ],
            "name": "underCollateralized",
            "outputs": [{"internalType": "bool", "name": "", "type": "bool"}],
            "stateMutability": "pure",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "uint256", "name": "amount", "type": "uint256"}
            ],
            "name": "withdraw",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
    ],
)



"""LiquidLogic contract binding and data structures."""

# This module has been generated using pyabigen v0.2.11

import typing

import eth_typing
import web3
from web3.contract import contract

__version__ = "v1.0.2-alpha"


class LiquidLogic:
    """LiquidLogic contract binding.

    Parameters
    ----------
    w3 : web3.Web3
    address : eth_typing.ChecksumAddress
        The address of a deployed LiquidLogic contract.
    """

    _contract: contract.Contract

    def __init__(
        self,
        w3: web3.Web3,
        address: eth_typing.ChecksumAddress,
    ):
        self._contract = w3.eth.contract(
            address=address,
            abi=ABI,
        )

    @property
    def Approval(self) -> contract.ContractEvent:
        """Binding for `event Approval` on the LiquidLogic contract."""
        return self._contract.events.Approval

    @property
    def Transfer(self) -> contract.ContractEvent:
        """Binding for `event Transfer` on the LiquidLogic contract."""
        return self._contract.events.Transfer

    def commission_rate_decimals(
        self,
    ) -> int:
        """Binding for `COMMISSION_RATE_DECIMALS` on the LiquidLogic contract.

        Returns
        -------
        int
        """
        return_value = self._contract.functions.COMMISSION_RATE_DECIMALS().call()
        return int(return_value)

    def commission_rate_scale_factor(
        self,
    ) -> int:
        """Binding for `COMMISSION_RATE_SCALE_FACTOR` on the LiquidLogic contract.

        Returns
        -------
        int
        """
        return_value = self._contract.functions.COMMISSION_RATE_SCALE_FACTOR().call()
        return int(return_value)

    def fee_factor_unit_recip(
        self,
    ) -> int:
        """Binding for `FEE_FACTOR_UNIT_RECIP` on the LiquidLogic contract.

        Returns
        -------
        int
        """
        return_value = self._contract.functions.FEE_FACTOR_UNIT_RECIP().call()
        return int(return_value)

    def fee_factor_unit_recip_decimals(
        self,
    ) -> int:
        """Binding for `FEE_FACTOR_UNIT_RECIP_DECIMALS` on the LiquidLogic contract.

        Returns
        -------
        int
        """
        return_value = self._contract.functions.FEE_FACTOR_UNIT_RECIP_DECIMALS().call()
        return int(return_value)

    def allowance(
        self,
        _owner: eth_typing.ChecksumAddress,
        _spender: eth_typing.ChecksumAddress,
    ) -> int:
        """Binding for `allowance` on the LiquidLogic contract.

        See {IERC20-allowance}.

        Parameters
        ----------
        _owner : eth_typing.ChecksumAddress
        _spender : eth_typing.ChecksumAddress

        Returns
        -------
        int
        """
        return_value = self._contract.functions.allowance(
            _owner,
            _spender,
        ).call()
        return int(return_value)

    def approve(
        self,
        _spender: eth_typing.ChecksumAddress,
        _amount: int,
    ) -> contract.ContractFunction:
        """Binding for `approve` on the LiquidLogic contract.

        Parameters
        ----------
        _spender : eth_typing.ChecksumAddress
        _amount : int

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.approve(
            _spender,
            _amount,
        )

    def balance_of(
        self,
        _delegator: eth_typing.ChecksumAddress,
    ) -> int:
        """Binding for `balanceOf` on the LiquidLogic contract.

        Returns the amount of liquid newtons held by the account (ERC-20).

        Parameters
        ----------
        _delegator : eth_typing.ChecksumAddress

        Returns
        -------
        int
        """
        return_value = self._contract.functions.balanceOf(
            _delegator,
        ).call()
        return int(return_value)

    def claim_rewards(
        self,
    ) -> contract.ContractFunction:
        """Binding for `claimRewards` on the LiquidLogic contract.

        Withdraws all fees earned so far by the caller.

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.claimRewards()

    def claim_treasury_atn(
        self,
    ) -> contract.ContractFunction:
        """Binding for `claimTreasuryATN` on the LiquidLogic contract.

        Send the unclaimed ATN entitled to treasury to treasury account

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.claimTreasuryATN()

    def decimals(
        self,
    ) -> int:
        """Binding for `decimals` on the LiquidLogic contract.

        Returns
        -------
        int
            uint8 the number of decimals the LNTN token uses.
        """
        return_value = self._contract.functions.decimals().call()
        return int(return_value)

    def get_commission_rate(
        self,
    ) -> int:
        """Binding for `getCommissionRate` on the LiquidLogic contract.

        Returns
        -------
        int
        """
        return_value = self._contract.functions.getCommissionRate().call()
        return int(return_value)

    def get_treasury(
        self,
    ) -> eth_typing.ChecksumAddress:
        """Binding for `getTreasury` on the LiquidLogic contract.

        Returns
        -------
        eth_typing.ChecksumAddress
        """
        return_value = self._contract.functions.getTreasury().call()
        return eth_typing.ChecksumAddress(return_value)

    def get_treasury_unclaimed_atn(
        self,
    ) -> int:
        """Binding for `getTreasuryUnclaimedATN` on the LiquidLogic contract.

        Returns the ATN amount that is yet to claim by treasury. Call function
        `claimTreasuryATN()` to claim.

        Returns
        -------
        int
        """
        return_value = self._contract.functions.getTreasuryUnclaimedATN().call()
        return int(return_value)

    def get_validator(
        self,
    ) -> eth_typing.ChecksumAddress:
        """Binding for `getValidator` on the LiquidLogic contract.

        Returns
        -------
        eth_typing.ChecksumAddress
        """
        return_value = self._contract.functions.getValidator().call()
        return eth_typing.ChecksumAddress(return_value)

    def locked_balance_of(
        self,
        _delegator: eth_typing.ChecksumAddress,
    ) -> int:
        """Binding for `lockedBalanceOf` on the LiquidLogic contract.

        Returns the amount of locked liquid newtons held by the account.

        Parameters
        ----------
        _delegator : eth_typing.ChecksumAddress

        Returns
        -------
        int
        """
        return_value = self._contract.functions.lockedBalanceOf(
            _delegator,
        ).call()
        return int(return_value)

    def name(
        self,
    ) -> str:
        """Binding for `name` on the LiquidLogic contract.

        Returns
        -------
        str
        """
        return_value = self._contract.functions.name().call()
        return str(return_value)

    def symbol(
        self,
    ) -> str:
        """Binding for `symbol` on the LiquidLogic contract.

        Returns
        -------
        str
        """
        return_value = self._contract.functions.symbol().call()
        return str(return_value)

    def total_supply(
        self,
    ) -> int:
        """Binding for `totalSupply` on the LiquidLogic contract.

        Returns the total amount of stake token issued.

        Returns
        -------
        int
        """
        return_value = self._contract.functions.totalSupply().call()
        return int(return_value)

    def transfer(
        self,
        _to: eth_typing.ChecksumAddress,
        _amount: int,
    ) -> contract.ContractFunction:
        """Binding for `transfer` on the LiquidLogic contract.

        Moves `_amount` LNEW tokens from the caller's account to the recipient `_to`.

        Parameters
        ----------
        _to : eth_typing.ChecksumAddress
        _amount : int

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.transfer(
            _to,
            _amount,
        )

    def transfer_from(
        self,
        _sender: eth_typing.ChecksumAddress,
        _recipient: eth_typing.ChecksumAddress,
        _amount: int,
    ) -> contract.ContractFunction:
        """Binding for `transferFrom` on the LiquidLogic contract.

        Parameters
        ----------
        _sender : eth_typing.ChecksumAddress
        _recipient : eth_typing.ChecksumAddress
        _amount : int

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.transferFrom(
            _sender,
            _recipient,
            _amount,
        )

    def unclaimed_rewards(
        self,
        _account: eth_typing.ChecksumAddress,
    ) -> int:
        """Binding for `unclaimedRewards` on the LiquidLogic contract.

        Calculates the total claimable fees (ATN) earned by the delegator to-date.

        Parameters
        ----------
        _account : eth_typing.ChecksumAddress
            Delegator account.

        Returns
        -------
        int
        """
        return_value = self._contract.functions.unclaimedRewards(
            _account,
        ).call()
        return int(return_value)

    def unlocked_balance_of(
        self,
        _delegator: eth_typing.ChecksumAddress,
    ) -> int:
        """Binding for `unlockedBalanceOf` on the LiquidLogic contract.

        Returns the amount of unlocked liquid newtons held by the account.

        Parameters
        ----------
        _delegator : eth_typing.ChecksumAddress

        Returns
        -------
        int
        """
        return_value = self._contract.functions.unlockedBalanceOf(
            _delegator,
        ).call()
        return int(return_value)


ABI = typing.cast(
    eth_typing.ABI,
    [
        {"inputs": [], "stateMutability": "nonpayable", "type": "constructor"},
        {
            "anonymous": False,
            "inputs": [
                {
                    "indexed": True,
                    "internalType": "address",
                    "name": "owner",
                    "type": "address",
                },
                {
                    "indexed": True,
                    "internalType": "address",
                    "name": "spender",
                    "type": "address",
                },
                {
                    "indexed": False,
                    "internalType": "uint256",
                    "name": "value",
                    "type": "uint256",
                },
            ],
            "name": "Approval",
            "type": "event",
        },
        {
            "anonymous": False,
            "inputs": [
                {
                    "indexed": True,
                    "internalType": "address",
                    "name": "from",
                    "type": "address",
                },
                {
                    "indexed": True,
                    "internalType": "address",
                    "name": "to",
                    "type": "address",
                },
                {
                    "indexed": False,
                    "internalType": "uint256",
                    "name": "value",
                    "type": "uint256",
                },
            ],
            "name": "Transfer",
            "type": "event",
        },
        {"stateMutability": "payable", "type": "fallback"},
        {
            "inputs": [],
            "name": "COMMISSION_RATE_DECIMALS",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "COMMISSION_RATE_SCALE_FACTOR",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "FEE_FACTOR_UNIT_RECIP",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "FEE_FACTOR_UNIT_RECIP_DECIMALS",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "_owner", "type": "address"},
                {"internalType": "address", "name": "_spender", "type": "address"},
            ],
            "name": "allowance",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "_spender", "type": "address"},
                {"internalType": "uint256", "name": "_amount", "type": "uint256"},
            ],
            "name": "approve",
            "outputs": [{"internalType": "bool", "name": "", "type": "bool"}],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "_delegator", "type": "address"}
            ],
            "name": "balanceOf",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "_account", "type": "address"},
                {"internalType": "uint256", "name": "_amount", "type": "uint256"},
            ],
            "name": "burn",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "claimRewards",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "claimTreasuryATN",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "decimals",
            "outputs": [{"internalType": "uint8", "name": "", "type": "uint8"}],
            "stateMutability": "pure",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "getCommissionRate",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "getTreasury",
            "outputs": [{"internalType": "address", "name": "", "type": "address"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "getTreasuryUnclaimedATN",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "getValidator",
            "outputs": [{"internalType": "address", "name": "", "type": "address"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "_account", "type": "address"},
                {"internalType": "uint256", "name": "_amount", "type": "uint256"},
            ],
            "name": "lock",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "_delegator", "type": "address"}
            ],
            "name": "lockedBalanceOf",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "_account", "type": "address"},
                {"internalType": "uint256", "name": "_amount", "type": "uint256"},
            ],
            "name": "mint",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "name",
            "outputs": [{"internalType": "string", "name": "", "type": "string"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "uint256", "name": "_ntnReward", "type": "uint256"}
            ],
            "name": "redistribute",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "payable",
            "type": "function",
        },
        {
            "inputs": [{"internalType": "uint256", "name": "_rate", "type": "uint256"}],
            "name": "setCommissionRate",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "symbol",
            "outputs": [{"internalType": "string", "name": "", "type": "string"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "totalSupply",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "_to", "type": "address"},
                {"internalType": "uint256", "name": "_amount", "type": "uint256"},
            ],
            "name": "transfer",
            "outputs": [{"internalType": "bool", "name": "_success", "type": "bool"}],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "_sender", "type": "address"},
                {"internalType": "address", "name": "_recipient", "type": "address"},
                {"internalType": "uint256", "name": "_amount", "type": "uint256"},
            ],
            "name": "transferFrom",
            "outputs": [{"internalType": "bool", "name": "", "type": "bool"}],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "_account", "type": "address"}
            ],
            "name": "unclaimedRewards",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "_account", "type": "address"},
                {"internalType": "uint256", "name": "_amount", "type": "uint256"},
            ],
            "name": "unlock",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "_delegator", "type": "address"}
            ],
            "name": "unlockedBalanceOf",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {"stateMutability": "payable", "type": "receive"},
    ],
)



"""Essential protocol parameters exposed as Python variables for convenience."""

from typing import cast

from eth_typing import ChecksumAddress

AUTONITY_CONTRACT_ADDRESS = cast(
    ChecksumAddress, "0xBd770416a3345F91E4B34576cb804a576fa48EB1"
)
AUTONITY_CONTRACT_VERSION = 1

NATIVE_TOKEN_SYMBOL = "ATN"
NATIVE_TOKEN_DECIMALS = 18



"""ACU contract binding and data structures."""

# This module has been generated using pyabigen v0.2.11

import typing

import eth_typing
import web3
from web3.contract import contract

__version__ = "v1.0.2-alpha"


class ACU:
    """ACU contract binding.

    Computes the value of the ACU, an optimal currency basket of 7 free-floating fiat
    currencies.

    Parameters
    ----------
    w3 : web3.Web3
    address : eth_typing.ChecksumAddress
        The address of a deployed ACU contract.
    """

    _contract: contract.Contract

    def __init__(
        self,
        w3: web3.Web3,
        address: eth_typing.ChecksumAddress,
    ):
        self._contract = w3.eth.contract(
            address=address,
            abi=ABI,
        )

    @property
    def BasketModified(self) -> contract.ContractEvent:
        """Binding for `event BasketModified` on the ACU contract.

        The ACU symbols, quantites, or scale were modified.
        """
        return self._contract.events.BasketModified

    @property
    def Updated(self) -> contract.ContractEvent:
        """Binding for `event Updated` on the ACU contract.

        The ACU value was updated.
        """
        return self._contract.events.Updated

    def modify_basket(
        self,
        symbols_: typing.List[str],
        quantities_: typing.List[int],
        scale_: int,
    ) -> contract.ContractFunction:
        """Binding for `modifyBasket` on the ACU contract.

        Modify the ACU symbols, quantites, or scale.

        Parameters
        ----------
        symbols_ : typing.List[str]
            The symbols used to retrieve prices
        quantities_ : typing.List[int]
            The basket quantity corresponding to each symbol
        scale_ : int
            The scale for quantities and the ACU value

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.modifyBasket(
            symbols_,
            quantities_,
            scale_,
        )

    def quantities(
        self,
    ) -> typing.List[int]:
        """Binding for `quantities` on the ACU contract.

        The basket quantities that are used to compute the ACU.

        Returns
        -------
        typing.List[int]
            Array of quantities
        """
        return_value = self._contract.functions.quantities().call()
        return [int(return_value_elem) for return_value_elem in return_value]

    def round(
        self,
    ) -> int:
        """Binding for `round` on the ACU contract.

        The Oracle round of the current ACU value.

        Returns
        -------
        int
        """
        return_value = self._contract.functions.round().call()
        return int(return_value)

    def scale(
        self,
    ) -> int:
        """Binding for `scale` on the ACU contract.

        The decimal places used to represent the ACU as a fixed-point integer. It is
        also the scale used to represent the basket quantities.

        Returns
        -------
        int
        """
        return_value = self._contract.functions.scale().call()
        return int(return_value)

    def scale_factor(
        self,
    ) -> int:
        """Binding for `scaleFactor` on the ACU contract.

        The multiplier for scaling numbers to the ACU scaled representation.

        Returns
        -------
        int
        """
        return_value = self._contract.functions.scaleFactor().call()
        return int(return_value)

    def symbols(
        self,
    ) -> typing.List[str]:
        """Binding for `symbols` on the ACU contract.

        The symbols that are used to compute the ACU.

        Returns
        -------
        typing.List[str]
            Array of symbols
        """
        return_value = self._contract.functions.symbols().call()
        return [str(return_value_elem) for return_value_elem in return_value]

    def value(
        self,
    ) -> int:
        """Binding for `value` on the ACU contract.

        The latest ACU value that was computed.

        Returns
        -------
        int
            ACU value in fixed-point integer representation
        """
        return_value = self._contract.functions.value().call()
        return int(return_value)


ABI = typing.cast(
    eth_typing.ABI,
    [
        {
            "inputs": [
                {"internalType": "string[]", "name": "symbols_", "type": "string[]"},
                {
                    "internalType": "uint256[]",
                    "name": "quantities_",
                    "type": "uint256[]",
                },
                {"internalType": "uint256", "name": "scale_", "type": "uint256"},
                {"internalType": "address", "name": "autonity", "type": "address"},
                {"internalType": "address", "name": "operator", "type": "address"},
                {"internalType": "address", "name": "oracle", "type": "address"},
            ],
            "stateMutability": "nonpayable",
            "type": "constructor",
        },
        {"inputs": [], "name": "InvalidBasket", "type": "error"},
        {"inputs": [], "name": "NoACUValue", "type": "error"},
        {"inputs": [], "name": "Unauthorized", "type": "error"},
        {
            "anonymous": False,
            "inputs": [
                {
                    "indexed": False,
                    "internalType": "string[]",
                    "name": "symbols",
                    "type": "string[]",
                },
                {
                    "indexed": False,
                    "internalType": "uint256[]",
                    "name": "quantities",
                    "type": "uint256[]",
                },
                {
                    "indexed": False,
                    "internalType": "uint256",
                    "name": "scale",
                    "type": "uint256",
                },
            ],
            "name": "BasketModified",
            "type": "event",
        },
        {
            "anonymous": False,
            "inputs": [
                {
                    "indexed": False,
                    "internalType": "uint256",
                    "name": "height",
                    "type": "uint256",
                },
                {
                    "indexed": False,
                    "internalType": "uint256",
                    "name": "timestamp",
                    "type": "uint256",
                },
                {
                    "indexed": False,
                    "internalType": "uint256",
                    "name": "round",
                    "type": "uint256",
                },
                {
                    "indexed": False,
                    "internalType": "int256",
                    "name": "value",
                    "type": "int256",
                },
            ],
            "name": "Updated",
            "type": "event",
        },
        {
            "inputs": [
                {"internalType": "string[]", "name": "symbols_", "type": "string[]"},
                {
                    "internalType": "uint256[]",
                    "name": "quantities_",
                    "type": "uint256[]",
                },
                {"internalType": "uint256", "name": "scale_", "type": "uint256"},
            ],
            "name": "modifyBasket",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "quantities",
            "outputs": [{"internalType": "uint256[]", "name": "", "type": "uint256[]"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "round",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "scale",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "scaleFactor",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "operator", "type": "address"}
            ],
            "name": "setOperator",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "oracle", "type": "address"}
            ],
            "name": "setOracle",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "symbols",
            "outputs": [{"internalType": "string[]", "name": "", "type": "string[]"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "update",
            "outputs": [{"internalType": "bool", "name": "status", "type": "bool"}],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "value",
            "outputs": [{"internalType": "int256", "name": "", "type": "int256"}],
            "stateMutability": "view",
            "type": "function",
        },
    ],
)



"""UpgradeManager contract binding and data structures."""

# This module has been generated using pyabigen v0.2.11

import typing

import eth_typing
import web3
from web3.contract import contract

__version__ = "v1.0.2-alpha"


class UpgradeManager:
    """UpgradeManager contract binding.

    Parameters
    ----------
    w3 : web3.Web3
    address : eth_typing.ChecksumAddress
        The address of a deployed UpgradeManager contract.
    """

    _contract: contract.Contract

    def __init__(
        self,
        w3: web3.Web3,
        address: eth_typing.ChecksumAddress,
    ):
        self._contract = w3.eth.contract(
            address=address,
            abi=ABI,
        )

    def autonity(
        self,
    ) -> eth_typing.ChecksumAddress:
        """Binding for `autonity` on the UpgradeManager contract.

        Returns
        -------
        eth_typing.ChecksumAddress
        """
        return_value = self._contract.functions.autonity().call()
        return eth_typing.ChecksumAddress(return_value)

    def operator(
        self,
    ) -> eth_typing.ChecksumAddress:
        """Binding for `operator` on the UpgradeManager contract.

        Returns
        -------
        eth_typing.ChecksumAddress
        """
        return_value = self._contract.functions.operator().call()
        return eth_typing.ChecksumAddress(return_value)

    def upgrade(
        self,
        _target: eth_typing.ChecksumAddress,
        _data: str,
    ) -> contract.ContractFunction:
        """Binding for `upgrade` on the UpgradeManager contract.

        Parameters
        ----------
        _target : eth_typing.ChecksumAddress
            is the target contract address to be updated.
        _data : str
            is the contract creation code.

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.upgrade(
            _target,
            _data,
        )


ABI = typing.cast(
    eth_typing.ABI,
    [
        {
            "inputs": [
                {"internalType": "address", "name": "_autonity", "type": "address"},
                {"internalType": "address", "name": "_operator", "type": "address"},
            ],
            "stateMutability": "nonpayable",
            "type": "constructor",
        },
        {
            "inputs": [],
            "name": "autonity",
            "outputs": [{"internalType": "address", "name": "", "type": "address"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "operator",
            "outputs": [{"internalType": "address", "name": "", "type": "address"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "_account", "type": "address"}
            ],
            "name": "setOperator",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "_target", "type": "address"},
                {"internalType": "string", "name": "_data", "type": "string"},
            ],
            "name": "upgrade",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
    ],
)



"""Package for interacting with the protocol contracts of the Autonity network."""

from . import constants, contracts
from .factory import (
    Accountability,
    ACU,
    Autonity,
    InflationController,
    LiquidLogic,
    OmissionAccountability,
    Oracle,
    Stabilization,
    SupplyControl,
    UpgradeManager,
)

__all__ = (
    "constants",
    "contracts",
    "Accountability",
    "ACU",
    "Autonity",
    "InflationController",
    "LiquidLogic",
    "OmissionAccountability",
    "Oracle",
    "Stabilization",
    "SupplyControl",
    "UpgradeManager",
)



"""SupplyControl contract binding and data structures."""

# This module has been generated using pyabigen v0.2.11

import typing

import eth_typing
import web3
from web3.contract import contract

__version__ = "v1.0.2-alpha"


class SupplyControl:
    """SupplyControl contract binding.

    Controls the supply of Auton on the network.

    Parameters
    ----------
    w3 : web3.Web3
    address : eth_typing.ChecksumAddress
        The address of a deployed SupplyControl contract.
    """

    _contract: contract.Contract

    def __init__(
        self,
        w3: web3.Web3,
        address: eth_typing.ChecksumAddress,
    ):
        self._contract = w3.eth.contract(
            address=address,
            abi=ABI,
        )

    @property
    def Burn(self) -> contract.ContractEvent:
        """Binding for `event Burn` on the SupplyControl contract.

        Auton was burned.
        """
        return self._contract.events.Burn

    @property
    def Mint(self) -> contract.ContractEvent:
        """Binding for `event Mint` on the SupplyControl contract.

        Auton was minted.
        """
        return self._contract.events.Mint

    def available_supply(
        self,
    ) -> int:
        """Binding for `availableSupply` on the SupplyControl contract.

        The supply of Auton available for minting.

        Returns
        -------
        int
        """
        return_value = self._contract.functions.availableSupply().call()
        return int(return_value)

    def set_stabilizer(
        self,
        stabilizer_: eth_typing.ChecksumAddress,
    ) -> contract.ContractFunction:
        """Binding for `setStabilizer` on the SupplyControl contract.

        Update the stabilizer that is authorized to mint and burn.

        Parameters
        ----------
        stabilizer_ : eth_typing.ChecksumAddress
            The new stabilizer account

        Returns
        -------
        web3.contract.contract.ContractFunction
            A contract function instance to be sent in a transaction.
        """
        return self._contract.functions.setStabilizer(
            stabilizer_,
        )

    def stabilizer(
        self,
    ) -> eth_typing.ChecksumAddress:
        """Binding for `stabilizer` on the SupplyControl contract.

        The account that is authorized to mint and burn.

        Returns
        -------
        eth_typing.ChecksumAddress
        """
        return_value = self._contract.functions.stabilizer().call()
        return eth_typing.ChecksumAddress(return_value)

    def total_supply(
        self,
    ) -> int:
        """Binding for `totalSupply` on the SupplyControl contract.

        The total supply of Auton under management.

        Returns
        -------
        int
        """
        return_value = self._contract.functions.totalSupply().call()
        return int(return_value)


ABI = typing.cast(
    eth_typing.ABI,
    [
        {
            "inputs": [
                {"internalType": "address", "name": "autonity", "type": "address"},
                {"internalType": "address", "name": "operator", "type": "address"},
                {"internalType": "address", "name": "stabilizer_", "type": "address"},
            ],
            "stateMutability": "payable",
            "type": "constructor",
        },
        {"inputs": [], "name": "InvalidAmount", "type": "error"},
        {"inputs": [], "name": "InvalidRecipient", "type": "error"},
        {"inputs": [], "name": "Unauthorized", "type": "error"},
        {"inputs": [], "name": "ZeroValue", "type": "error"},
        {
            "anonymous": False,
            "inputs": [
                {
                    "indexed": False,
                    "internalType": "uint256",
                    "name": "amount",
                    "type": "uint256",
                }
            ],
            "name": "Burn",
            "type": "event",
        },
        {
            "anonymous": False,
            "inputs": [
                {
                    "indexed": False,
                    "internalType": "address",
                    "name": "recipient",
                    "type": "address",
                },
                {
                    "indexed": False,
                    "internalType": "uint256",
                    "name": "amount",
                    "type": "uint256",
                },
            ],
            "name": "Mint",
            "type": "event",
        },
        {
            "inputs": [],
            "name": "availableSupply",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "burn",
            "outputs": [],
            "stateMutability": "payable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "recipient", "type": "address"},
                {"internalType": "uint256", "name": "amount", "type": "uint256"},
            ],
            "name": "mint",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "operator", "type": "address"}
            ],
            "name": "setOperator",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [
                {"internalType": "address", "name": "stabilizer_", "type": "address"}
            ],
            "name": "setStabilizer",
            "outputs": [],
            "stateMutability": "nonpayable",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "stabilizer",
            "outputs": [{"internalType": "address", "name": "", "type": "address"}],
            "stateMutability": "view",
            "type": "function",
        },
        {
            "inputs": [],
            "name": "totalSupply",
            "outputs": [{"internalType": "uint256", "name": "", "type": "uint256"}],
            "stateMutability": "view",
            "type": "function",
        },
    ],
)
